from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("clk200", 0,
        Subsignal("p", Pins("H9"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("G9"), IOStandard("DIFF_SSTL15"))
    ),

    ("user_led", 0, Pins("Y21"), IOStandard("LVCMOS25")),
    ("user_led", 1, Pins("G2"), IOStandard("LVCMOS15")),
    ("user_led", 2, Pins("W21"), IOStandard("LVCMOS25")),
    ("user_led", 3, Pins("A17"), IOStandard("LVCMOS15")),

    ("serial", 0,
        Subsignal("tx", Pins("C19")),
        Subsignal("rx", Pins("D18")),
        IOStandard("LVCMOS18")),

    ("user_sma_clock", 0,
        Subsignal("p", Pins("AD18"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE")),
        Subsignal("n", Pins("AD19"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE"))
    ),

    ("user_sma_mgt", 0,
        Subsignal("txp", Pins("Y2")),
        Subsignal("txn", Pins("Y1")),
        Subsignal("rxp", Pins("AB6")),
        Subsignal("rxn", Pins("AB5"))  
    ),

    ("sfp", 0,
        Subsignal("txp", Pins("W4")),
        Subsignal("txn", Pins("W3")),
        Subsignal("rxp", Pins("Y6")),
        Subsignal("rxn", Pins("Y5"))
    ),
    ("sfp_tx_disable_n", 0, Pins("AA18"), IOStandard("LVCMOS25")),

    ("si5324", 0,
        Subsignal("rst_n", Pins("W23"), IOStandard("LVCMOS25")),
        Subsignal("int", Pins("AJ25"), IOStandard("LVCMOS25"))
    ),
    ("si5324_clkin", 0,
        Subsignal("p", Pins("AD20"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("AE20"), IOStandard("LVDS_25"))
    ),
    ("si5324_clkout", 0,
        Subsignal("p", Pins("AC8")),
        Subsignal("n", Pins("AC7"))
    ),

    ("pmod1", 0, Pins("AJ21"), IOStandard("LVCMOS25")),
    ("pmod1", 1, Pins("AK21"), IOStandard("LVCMOS25")),
    ("pmod1", 2, Pins("AB21"), IOStandard("LVCMOS25")),
    ("pmod1", 3, Pins("AB16"), IOStandard("LVCMOS25")),
    ("pmod1", 4, Pins("Y20"), IOStandard("LVCMOS25")),
    ("pmod1", 5, Pins("AA20"), IOStandard("LVCMOS25")),
    ("pmod1", 6, Pins("AC18"), IOStandard("LVCMOS25")),
    ("pmod1", 7, Pins("AC19"), IOStandard("LVCMOS25")),

]

_connectors = [
    ("HPC", {
        # ZC706 User Guide p. 66 col. 1
        "DP1_M2C_P": "AJ8",
        "DP2_M2C_N": "AJ7",
        "DP2_M2C_P": "AG8",
        "DP2_M2C_N": "AG7",
        "DP3_M2C_P": "AE8",
        "DP3_M2C_N": "AE7",
        "DP4_M2C_P": "AH6",
        "DP4_M2C_N": "AH5",
        "DP5_M2C_P": "AG4",
        "DP5_M2C_N": "AG3",
        "DP1_C2M_P": "AK6",
        "DP1_C2M_N": "AK5",
        "DP2_C2M_P": "AJ4",
        "DP2_C2M_N": "AJ3",
        "DP3_C2M_P": "AK2",
        "DP3_C2M_N": "AK1",
        "DP4_C2M_P": "AH2",
        "DP4_C2M_N": "AH1",
        "DP5_C2M_P": "AF2",
        "DP5_C2M_N": "AF1",
        "DP0_C2M_P": "AK10",
        "DP0_C2M_N": "AK9",
        "DP0_M2C_P": "AH10",
        "DP0_M2C_N": "AH9",
        "LA06_P": "AG22",
        "LA06_N": "AH22",
        # ZC706 User Guide p. 66 col. 2
        "DP7_M2C_P": "AD6",
        "DP7_M2C_N": "AD5",
        "DP6_M2C_P": "AF6",
        "DP6_M2C_N": "AF5",
        "GBTCLK1_M2C_P": "AA8",
        "GBTCLK1_M2C_N": "AA7",
        "DP7_C2M_P": "AD2",
        "DP7_C2M_N": "AD1",
        "DP6_C2M_P": "AE4",
        "DP6_C2M_N": "AE3",
        "GBTCLK0_M2C_P": "AD10",
        "GBTCLK0_M2C_N": "AD9",
        "LA01_CC_P": "AG21",
        "LA01_CC_N": "AH21",
        "LA05_P": "AH23",
        # ZC706 User Guide p. 67 col. 1
        "LA10_P": "AG24",
        "LA10_N": "AG25",
        "LA14_P": "AC24",
        "LA14_N": "AD24",
        "LA18_CC_P": "W25",
        "LA18_CC_N": "W26",
        "LA27_P": "V28",
        "LA27_N": "V29",
        "IIC_SCL": "U65.13",
        "IIC_SDA": "U65.12",
        # ZC706 User Guide p. 67 col. 2
        "LA05_N": "AH24",
        "LA09_P": "AD21",
        "LA09_N": "AE21",
        "LA13_P": "AA22",
        "LA13_N": "AA23",
        "LA17_CC_P": "V23",
        "LA17_CC_N": "W24",
        "LA23_P": "P25",
        "LA23_N": "P26",
        "LA26_P": "R28",
        "LA26_N": "T28",
        "PG_M2C": "U16.16",
        # ZC706 User Guide p. 68 col. 1
        "CLK1_M2C_P": "U26",
        "CLK1_M2C_N": "U27",
        "LA00_CC_P": "AF20",
        "LA00_CC_N": "AG20",
        "LA03_P": "AH19",
        "LA03_N": "AJ19",
        "LA08_P": "AF19",
        "LA08_N": "AG19",
        "LA12_P": "AF23",
        "LA12_N": "AF24",
        "LA16_P": "AA24",
        "LA16_N": "AB24",
        "LA20_P": "U25",
        "LA20_N": "V26",
        "LA22_P": "V27",
        "LA22_N": "W28",
        "LA25_P": "T29",
        "LA25_N": "U29",
        "LA29_P": "R25",
        "LA29_N": "R26",
        "LA31_P": "N29",
        "LA31_N": "P29",
        "LA33_P": "N26",
        "LA33_N": "N27",
        # ZC706 User Guide p. 68 col. 2
        "PRSNT_M2C_B": "U16.15",
        "CLK0_M2C_P": "AE22",
        "CLK0_M2C_N": "AF22",
        "LA02_P": "AK17",
        "LA02_N": "AK18",
        "LA04_P": "AJ20",
        "LA04_N": "AK20",
        "LA07_P": "AJ23",
        "LA07_N": "AJ24",
        "LA11_P": "AD23",
        "LA11_N": "AE23",
        "LA15_P": "Y22",
        "LA15_N": "Y23",
        "LA19_P": "T24",
        "LA19_N": "T25",
        "LA21_P": "W29",
        "LA21_N": "W30",
        "LA24_P": "T30",
        "LA24_N": "U30",
        "LA28_P": "P30",
        "LA28_N": "R30",
        "LA30_P": "P23",
        "LA30_N": "P24",
        "LA32_P": "P21",
        # ZC706 User Guide p. 69 col. 2
        "LA32_N": "R21",
        }
    ),
    ("LPC", {
        # ZC706 User Guide p. 70 col. 1
        "DP0_C2M_P": "AB2",
        "DP0_C2M_N": "AB1",
        "DP0_M2C_P": "AC4",
        "DP0_M2C_N": "AC3",
        "LA06_P": "AB12",
        "LA06_N": "AC12",
        "LA10_P": "AC14",
        "LA10_N": "AC13",
        "LA14_P": "AF18",
        "LA14_N": "AF17",
        "LA18_CC_P": "AE27",
        "LA18_CC_N": "AF27",
        "LA27_P": "AJ28",
        "LA27_N": "AJ29",
        "IIC_SCL": "U65.15",
        "IIC_SDA": "U65.14",
        # ZC706 User Guide p. 70 col. 2
        "PG_C2M": "AB20",
        "GBTCLK0_M2C_P": "U8",
        "GBTCLK0_M2C_N": "U7",
        "LA01_CC_P": "AF15",
        "LA01_CC_N": "AG15",
        "LA05_P": "AE16",
        "LA05_N": "AE15",
        "LA09_P": "AH14",
        "LA09_N": "AH13",
        "LA13_P": "AH17",
        "LA13_N": "AH16",
        "LA17_CC_P": "AB27",
        "LA17_CC_N": "AC27",
        "LA23_P": "AJ26",
        "LA23_N": "AK26",
        "LA26_P": "AJ30",
        "LA26_N": "AK30",
        # ZC706 User Guide p. 71 col. 1
        "CLK1_M2C_P": "AC28",
        "CLK1_M2C_N": "AD28",
        "LA00_CC_P": "AE13",
        "LA00_CC_N": "AF13",
        "LA03_P": "AG12",
        "LA03_N": "AH12",
        "LA08_P": "AD14",
        "LA08_N": "AD13",
        "LA12_P": "AD16",
        "LA12_N": "AD15",
        "LA16_P": "AE18",
        "LA16_N": "AE17",
        "LA20_P": "AG26",
        "LA20_N": "AG27",
        "LA22_P": "AK27",
        "LA22_N": "AK28",
        "LA25_P": "AF29",
        "LA25_N": "AG29",
        "LA29_P": "AE25",
        "LA29_N": "AF25",
        "LA31_P": "AC29",
        "LA31_N": "AD29",
        "LA33_P": "Y30",
        "LA33_N": "AA30",
        # ZC706 User Guide p. 71 col. 2
        "PRSNT_M2C_B": "U16.14",
        "CLK0_M2C_P": "AG17",
        "CLK0_M2C_N": "AG16",
        "LA02_P": "AE12",
        "LA02_N": "AF12",
        "LA04_P": "AJ15",
        "LA04_N": "AK15",
        "LA07_P": "AA15",
        "LA07_N": "AA14",
        "LA11_P": "AJ16",
        "LA11_N": "AK16",
        "LA15_P": "AB15",
        "LA15_N": "AB14",
        "LA19_P": "AH26",
        "LA19_N": "AH27",
        "LA21_P": "AH28",
        "LA21_N": "AH29",
        "LA24_P": "AF30",
        "LA24_N": "AG30",
        "LA28_P": "AD25",
        "LA28_N": "AE26",
        "LA30_P": "AB29",
        "LA30_N": "AB30",
        "LA32_P": "Y26",
        "LA32_N": "Y27",
        }
    ),
    ("XADC", {
        "GPIO0": "H14",
        "GPIO1": "J15",
        "GPIO2": "J16",
        "GPIO3": "J14",
        "VAUX0_N": "L14",
        "VAUX0_P": "L15",
        "VAUX8_N": "H13",
        "VAUX8_P": "J13",
        }
    ),
]

class Platform(XilinxPlatform):
    default_clk_name = "clk200"
    default_clk_period = 5  # 200 MHz

    def __init__(self):
        XilinxPlatform.__init__(self, "xc7z045-ffg900-2", _io, _connectors,
            toolchain="vivado")
